unit EITCheckListBox;
{ -------------------------------------------------------------------------------------}
{
    "EITCheckListBox" copyright 1999, Allen A. Maxwell.  All Rights Reserved.
    This component can be freely used and distributed in commercial and private
    environments, provided this notice is not modified in any way.
    Author's email address:  aamax@xmission.com  Allen A. Maxwell

    The Following is the disclaimer for the base class component which was written
    by Patrick Bisacier in 1996/97.  Got to give credit where credit is due...
    This component only slightly alters the origional - changing how multi and
    extended selecting is handled.

}
{ A "CheckListBox" component for Delphi32.                                             }
{ Copyright 1997, Patrick Brisacier.  All Rights Reserved.                             }
{ This component can be freely used and distributed in commercial and private          }
{ environments, provided this notice is not modified in any way.                       }
{ -------------------------------------------------------------------------------------}
{ Feel free to contact us if you have any questions, comments or suggestions at        }
{ PBrisacier@mail.dotcom.fr (Patrick Brisacier)                                        }
{ -------------------------------------------------------------------------------------}
{ Date last modified:  07/05/96                                                        }
{ -------------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, PBCheckLB;

type
  TEITCheckListBox = class(TPBCheckListBox)
  private
    { Private declarations }
    { Test if the mouse click hit the checkboxes }
    FLastSelected: Integer;  // used for selecting blocks...
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                                                    X, Y: Integer); override;
  public
    { Public declarations }
    { constructor }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
  end;

procedure Register;

implementation

{ TEITCheckListBox }
{
   The majority of the functionality of the checklistbox is handled in the
   base class.  The only changes / extensions of this are the trapping of
   extended and multiselcting checks and subsequent handling of the group
   of selected items as a whole...
}
constructor TEITCheckListBox.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    FLastSelected := -1;
end;

procedure TEITCheckListBox.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
	Index, iCnt: Integer;
	Rect: TRect;
begin
	Index := ItemAtPos(Point(X, Y), True);
    // catch extended selecting state for special handling
    if (ssShift in Shift) and Multiselect and Extendedselect and
                              (FLastSelected <> -1) and (Index <> -1) then begin
        Rect := ItemRect(Index);
        if (Button = mbLeft)
                and (X >= Rect.Left + FOffset)
                and (X < Rect.Left + FOffset +
                (Rect.Bottom - Rect.Top - 2)) then begin
            if FLastSelected < Index then begin
                // handle block with for statement using "TO"
                for iCnt := FLastSelected to (index - 1) do begin
                    case State[Index] of
                        // depending on state, set group except current item
                        // which is handled in the inherited code
                        cbUnchecked: begin
                            if FAllowGrayed then begin
                                State[iCnt] := cbGrayed
                                end
                            else begin
                                State[iCnt] := cbChecked;
                            end;
                        end;
                        cbChecked: begin
                            State[iCnt] := cbUnchecked;
                        end;
                        cbGrayed: begin
                            State[iCnt] := cbChecked;
                        end;
                    end;
                    InvalidateCheck(iCnt);
                end;
                end
            else begin
                // handle block using "downto"
                for iCnt := FLastSelected downto (index - 1) do begin
                    case State[Index] of
                        // depending on state, set group except current item
                        // which is handled in the inherited code
                        cbUnchecked: begin
                            if FAllowGrayed then begin
                                State[iCnt] := cbGrayed
                                end
                            else begin
                                State[iCnt] := cbChecked;
                            end;
                        end;
                        cbChecked: begin
                            State[iCnt] := cbUnchecked;
                        end;
                        cbGrayed: begin
                            State[iCnt] := cbChecked;
                        end;
                    end;
                    InvalidateCheck(iCnt);
                end;
            end;
        end;
        end
    else if Index <> -1 then begin
        FLastSelected := Index;
    end;
	inherited;
end;

procedure Register;
begin
  RegisterComponents('BWI', [TEITCheckListBox]);
end;

end.
